/*
 * $Id: wakachi.c,v 1.1.1.1 2007/01/06 13:12:16 t-ishii Exp $
 *
 * Copyright (c) 2007	SRA Tohoku, Inc.
 * Copyright (c) 2007	SRA OSS, Inc. Japan
 *
 * Result of Open Source Software Development Activities of
 * Information-technology Promotion Agency, Japan.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose, without fee, and without a
 * written agreement is hereby granted, provided that the above
 * copyright notice and this paragraph and the following two
 * paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS
 * IS" BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE,
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include "postgres.h"

#include "funcapi.h"
#include "lib/stringinfo.h"
#include "utils/builtins.h"

#include "mecab.h"
#include "wakachi.h"

PG_MODULE_MAGIC;

PG_FUNCTION_INFO_V1(wakachi);

extern Datum wakachi(PG_FUNCTION_ARGS);

/* ----------
 * wakachi:
 * returns wakachi-kaki text
 *
 * C FUNCTION definition
 * wakachi(text) returns text
 *
 * TODO:
 *		normalization
 *		remove duplicate words
 *		remove non noun words
 * ----------
 */

#ifndef MECABDIC
#define MECABDIC "/usr/lib/mecab"
#endif

#ifdef MAXWAKACHILEN
#define	MAXWAKACHILEN 128
#endif

#define PG_TEXT_GET_STR(textp_) \
	DatumGetCString(DirectFunctionCall1(textout, PointerGetDatum(textp_)))
#define PG_STR_GET_TEXT(str_) \
	DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(str_)))
#define min(a, b) (a < b? a: b)

Datum
wakachi(PG_FUNCTION_ARGS)
{
	char buf[MAXWAKACHILEN+1];
	char *input;
	mecab_t *mecab;
	mecab_node_t *node;
	StringInfo string;
	text *result;
	int len;
	bool first = true;

	input = PG_TEXT_GET_STR(PG_GETARG_TEXT_P(0));
	mecab = mecab_new2(MECABDIC);

	if (mecab == NULL)
	{
		elog(ERROR, "mecab_new2 returns error");
	}

	node = mecab_sparse_tonode(mecab, input);

	string = makeStringInfo();
	initStringInfo(string);

	for (;  node; node = node->next)
	{
		if (node->stat != MECAB_BOS_NODE && node->stat != MECAB_EOS_NODE)
		{
			if (first == false)
				appendStringInfoString(string, " ");

			first = false;

			len = min(MAXWAKACHILEN, node->length);

			if (node->length > MAXWAKACHILEN)
				elog(ERROR, "wakachi: node->length(%d) too long. max: %d", node->length, MAXWAKACHILEN);

			strncpy(buf, node->surface, len);
			*(buf + len) = '\0';

			appendStringInfoString(string, buf);
		}
	}
	mecab_destroy(mecab);

	result = PG_STR_GET_TEXT(string->data);
	PG_RETURN_TEXT_P(result);
}
